function  [SBstructure] = DSmodelGeneration(independent, param, ic, algebraic, dxdt, dfdx, recnames, recformulas, tolerance)
% Generation of ODE m-file.
% input dxdt= f xconc
% output  dxdt = dfdx xconc + dfdz
% Paremeter setting
% ds/dt = M * s + W
% size s  = Ndep(Nind+Npar+Nini)
% size M  = Ndep(Nind+Npar+Nini)^2
% size W  = size s
[Ndep, Nind, Npar, Nini, GNind, GNpar, GNini] = check_number(independent, param, ic);
%% EQDSmodel(main)
sini = zeros(1,Ndep*(GNind+GNpar+GNini));
if GNini~=0
    c=1;
    for i=1:Nini
        if ic.index(i)~=0
            sini(1,Ndep*(GNind+GNpar+(c-1))+i) = 1;
            c=c+1;
        end
    end
end

SBstates = struct('name',{},'initialCondition',{},'ODE',{},'type',{},'compartment',{},'unittype',{},'notes',{});
SBalgebraic = struct('name',{},'formula',{},'initialCondition',{},'type',{},'compartment',{},'unittype',{},'notes',{});
SBparameters = struct('name',{},'value',{},'type',{},'compartment',{},'unittype',{},'notes',{});
SBvariables = struct('name',{},'formula',{},'type',{},'compartment',{},'unittype',{},'notes',{});
SBreactions = struct('name',{},'formula',{},'notes',{},'reversible',{},'fast',{});
SBfunctions = struct('name',{},'arguments',{},'formula',{},'notes',{});
eventassignmentStruct = struct('variable',{},'formula',{});
SBevents = struct('name',{},'trigger',{},'assignment',eventassignmentStruct,'notes',{});


% algebraic setting
if ~isempty(algebraic.name)
    for i=1:length(algebraic.name)
        SBvariables(i).name = char(algebraic.name(i));
        SBvariables(i).formula = char(algebraic.value(i));
    end
    c = i+1;
else
    c = 1;
end


%variables setting
Mflag = zeros(1,Ndep^2);
for i=1:Ndep^2
    if strcmp(char(dfdx(i)), num2str(0)) ==0
        flag = 0;        
        if flag == 0
            SBvariables(c).name = sprintf('M%d', i);
            SBvariables(c).formula = char(dfdx(i));
            c = c+1;
        else
            Mflag(i) = 1;
        end
    end
end

sODE = cell(Ndep*((GNind+GNpar+GNini)+1),1);
% species setting
for i=1:Ndep
    SBstates(i).name = char(ic.symbol(i));
    SBstates(i).initialCondition = ic.value(i);
    if strcmp(dxdt{i}, num2str(1))
        sODE{i} = '0';
    else
        sODE{i} = dxdt(i);
    end
end

% dynamic sensitivity setting
SBlength = Ndep;
for i=1:Ndep*(GNind+GNpar+GNini)
    SBstates(SBlength+i).name = sprintf('s%d',i);
    SBstates(SBlength+i).initialCondition = sini(i);
end

num = Ndep+1;
Snum = 1;
k=0;


for i=1:Ndep*(GNind+GNpar+GNini)
    sODE{num} = blanks(1);
    % Check Dynamic Sensitivity formulas
    for j=1:Ndep
        k = k+1;
        if strcmp(char(dfdx(k)), num2str(0)) ~= 1            
            if Mflag(k)==1
                sODE{num} = strcat(sODE{num},sprintf('+(%s*s%d)', char(dfdx(k)), Snum));
            else
                sODE{num} = strcat(sODE{num}, sprintf('+M%d*s%d', k, Snum));
            end
        end
        Snum = Snum+1;
    end
    
    if rem(i,Ndep) == 0
        k=0;
    else
        Snum = Snum-Ndep;
    end
    num = num+1;
end

for i=1:Ndep*(GNind+GNpar+GNini)
    sODE{i+Ndep} = strcat(sODE{i+Ndep}, sprintf('+W%d', i));
end

for i=1:length(sODE)
    SBstates(i).ODE = char(sODE{i});
end



%parameters setting
c=1;
for i=1:Nind
    if independent.index(i) ~= 0
        SBparameters(i).name = char(independent.symbol(i,1));
        SBparameters(i).value = independent.value(i,1);
        c = c+1;
    else
        SBparameters(i).name = char(independent.symbol(i,1));
        SBparameters(i).value = independent.value(i,1);
    end
end

for i=1:Npar
    par = i+Nind;
    if param.index(i,1) ~= 0
        SBparameters(par).name = char(param.symbol(i));
        SBparameters(par).value = param.value(i);
        c = c+1;
    else
        SBparameters(par).name = char(param.symbol(i,1));
        SBparameters(par).value = param.value(i,1);
    end
end


for i=1:length(recnames)
    SBreactions(i).name = recnames{i};
    SBreactions(i).formula = recformulas{i};
end

%event setting
eventassignmentStruct(1).formula = num2str(tolerance);
for i=1:Nini
    SBevents(i).name = sprintf('ev%d',i);
    SBevents(i).trigger = sprintf('lt(%s,%e)',ic.symbol{i},tolerance*0.1);
    eventassignmentStruct(1).variable = sprintf('%s',ic.symbol{i});
    SBevents(i).assignment = eventassignmentStruct;
end

% inputs substructure
inputsStruct = struct('name',{},'factors',{},'terms',{},'stateindex',{},'parindex',{});
% outputs substructure
outputsStruct = struct('name',{},'formula',{},'notes',{},'varindex',{});
% functions substructure
functionsStruct = struct('name',{},'arguments',{},'formula',{},'notes',{});
% states substructure
statesStruct = struct('name',{},'initialCondition',{},'ODE',{},'type',{},'compartment',{},'unittype',{},'notes',{});
% algebraic substructure
algebraicStruct = struct('name',{},'formula',{},'initialCondition',{},'type',{},'compartment',{},'unittype',{},'notes',{});
% parameters substructure
parametersStruct = struct('name',{},'value',{},'type',{},'compartment',{},'unittype',{},'notes',{});
% variables substructure
variablesStruct = struct('name',{},'formula',{},'type',{},'compartment',{},'unittype',{},'notes',{});
% reactions substructure
reactionsStruct = struct('name',{},'formula',{},'notes',{},'reversible',{},'fast',{});
% event assignment substructure
eventassignmentStruct = struct('variable',{},'formula',{});
% event substructure
eventStruct = struct('name',{},'trigger',{},'assignment',eventassignmentStruct,'notes',{});
% Create SBstructure
SBstructure = struct('name','unnamed_model','notes','','functions',functionsStruct,'states',statesStruct,'algebraic',algebraicStruct,'parameters',parametersStruct,'variables',variablesStruct,'reactions',reactionsStruct,'events',eventStruct,'functionsMATLAB','','inputs',inputsStruct,'outputs',outputsStruct);
SBstructure.states = SBstates;
SBstructure.algebraic = SBalgebraic;
SBstructure.parameters = SBparameters;
SBstructure.variables = SBvariables;
SBstructure.reactions = SBreactions;
SBstructure.functions = SBfunctions;
SBstructure.events = SBevents;









