function [independent, param, ic, algebraic, SBMLmodel] = DSparameterReader(filename, species, parameters, independents)
%% check on file form
[~,~,extension] = fileparts(filename);
if strcmp(extension, '.xml') || strcmp(extension, '.txt')
    [independent, param, ic, algebraic, SBMLmodel] = DSparameterReader_SBML(filename, species, parameters, independents);
else
    errorflag = 4;
    errorMsg([],errorflag);
end

%% Data storage
[Ndep, Nind, Npar, Nini, ~, ~, ~] = check_number(independent, param, ic);
if Ndep == 0
    errorMsg('dependent parameter', 1);
end
if Nind == 0
    errorMsg('independent parameter', 1);
end
if Npar == 0
    errorMsg('kinetic parameter', 1);
end
if Nini == 0
    errorMsg('iniital condition', 1);
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                          DSparameterReader_SBML                         % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [independent, param, ic, algebraic, SBMLmodel] = DSparameterReader_SBML(filename, species, parameters, independents)

SBMLmodel = SBmodel(filename);

% get initial value
iflag=0;
ic.symbol = SBstates(SBMLmodel);
ic.value = SBinitialconditions(SBMLmodel);
ic.value = ic.value';

Nini = length(ic.symbol);
ic.index = zeros(Nini,1);

if ~isempty(species)
    [m,n] = size(species);
    if n>m
        species = species';
    end
    
    for i=1:Nini
        for j=1:length(species)
            if strcmp(char(ic.symbol{i,1}), char(species(j,1)))
                ic.index(i,1) = 1;
                iflag=1;
            end
        end
    end
end

% get kinetic parameters
[param.symbol, param.value] = SBparameters(SBMLmodel);

Npar = length(param.symbol);
if iflag==0
    param.index = ones(Npar,1);
else
    param.index = zeros(Npar,1);
end

if ~isempty(parameters)
    param.index = zeros(Npar,1);
    [m,n] = size(parameters);
    if n>m
        parameters = parameters';
    end
    
    for i=1:Npar
        for j=1:length(parameters)
            if strcmp(char(param.symbol{i,1}), char(parameters{j,1}))
                param.index(i,1) = 1;
            end
        end
    end
end



% get independent parameters
[independent.symbol, indvalue] = SBvariables(SBMLmodel);
[algebraic.name, algebraic.value] = SBalgebraic(SBMLmodel);
if length(indvalue) > 1
    Nind = length(indvalue);
    checkInd = zeros(1,1);
    for i=1:Nind
        algeflag = 0;
        tf = isletter(indvalue{i,1});
        ind = find(tf);
        if length(ind) >= 1
            algeflag = 1;
        end
        if algeflag == 1
            checkInd(end+1,1) = i;
        end
    end
    checkInd(1,:) = [];
    Nalg = length(algebraic.name);
    for i=1:length(checkInd)
        algebraic.name{Nalg+1, 1} = independent.symbol{checkInd(i,1), 1};
        algebraic.value{Nalg+1,1} = indvalue{checkInd(i,1), 1};
        Nalg = Nalg+1;
    end
    independent.symbol(checkInd, :) = [];
    indvalue(checkInd,:) = [];
end

Nind = length(independent.symbol);
independent.index = zeros(Nind,1);


for i=1:Nind
    ind = str2double(cell2mat(indvalue(i,1)));
    independent.value(i,1) = ind;
end

if ~isempty(independents)
    [m,n] = size(independents);
    if n>m
        independents = independents';
    end
    for i=1:Nind
        for j=1:length(independents)
            if strcmp(char(independent.symbol{i,1}), char(independents{j,1})) || iflag==0;
                independent.index(i,1) = 1;
            end
        end
    end
end


end



