function [data] = DSsensitivity(filename, timevector, varargin)

warning('off','all');

tic;

[options, ode, species, parameters, independents, initial, debug, InH1, InH2, InH3] = CheckVarargin(varargin);

[independent, param, ic, algebraic, SBMLmodel] = DSparameterReader(filename, species, parameters, independents);
fprintf('elapsed time at prameterReader %fsec\n',toc);

if isempty(InH1)~=1
independent.value(4)=InH1;
end
if isempty(InH2)~=1
independent.value(5)=InH2;
end
if isempty(InH3)~=1
independent.value(6)=InH3;
end

[species, spformulas, recnames, recformulas] = DSGetRateLawsFromReactions(SBMLmodel);

[dxdt, dfdx, dfdz] = DScheckdae(independent, param, ic, algebraic, species, spformulas, recnames, recformulas);
fprintf('elapsed time at DSmodel for partial differential equation generation %fsec\n',toc);

% Dynamic sensitivity analysis
if ~isempty(debug)
    debug = filename;
end
[data] = DSsimulation(timevector, options, dxdt, dfdx, dfdz, independent, param, ic, algebraic, initial, recnames, recformulas, debug, ode);%fileō쐬EQDSmodel*
fprintf('simulation and estimation time %fsec\n',toc);
warning('on','all');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                  CheckVarargin                          % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [options, ode, species, parameters, independents, initial, debug, InH1, InH2, InH3] = CheckVarargin(info)

options = {};
ode = {};
species = {};
parameters = {};
independents = {};
initial = {};
debug = {};
InH1 = {};
InH2 = {};
InH3 = {};
Nargin = length(info);

if Nargin >=11
    error('There are too many input variables');
end
if Nargin >= 1
    options = info{1,1};
end
if Nargin >= 2
    ode = info{1,2};
end
if Nargin >= 3
    species = info{1,3};
end
if Nargin >= 4
    parameters = info{1,4};
end
if Nargin >= 5
    independents = info{1,5};
end
if Nargin >= 6
    initial = info{1,6};
end
if Nargin >= 7
    debug = info{1,7};
end
if Nargin >= 8
    InH1 = info{1,8};
end
if Nargin >= 9
    InH2 = info{1,9};
end
if Nargin >= 10
    InH3 = info{1,10};
end
