function [data] = DSsimulation(timevector, options, varargin)

if nargin == 14
    dxdt = varargin{1,1};
    dfdx = varargin{1,2};
    dfdz = varargin{1,3};
    independent = varargin{1,4};
    param = varargin{1,5};
    ic = varargin{1,6};
    algebraic = varargin{1,7};
    initial = varargin{1,8};
    recnames = varargin{1,9};
    recformulas = varargin{1,10};
    debug = varargin{1,11};
    ode = varargin{1,12};
else
    tic;
    load(varargin{1,1});
    debug ={};
end

[Ndep,Nind,Npar,Nini, GNind, GNpar, GNini] = check_number(independent, param, ic);

if length(timevector) == 1
    timevector = [0:timevector(end)];
end

if isempty(options)
    options = odeset('Abstol',1e-10,'Reltol',1e-10);
end

if isempty(ode)
    ode = 'ode15s';
end

if nargin==14
    tolerance = options.AbsTol;
    [SBstructure] = DSmodelGeneration(independent, param, ic, algebraic, dxdt, dfdx, recnames, recformulas, tolerance);
end
clear dfdx dxdt recformulas recnames;

if ~isempty(debug)
    [~,name,~] = fileparts(debug);
    save(sprintf('%s.mat',name),'SBstructure','dfdz','dxdt','dfdx', 'independent', 'param','ic',...
        'algebraic','initial','recnames','recformulas','ode');
end

GN = idivide(int32(120), Ndep);
if GN==0
    GN =1;
end
count = fix((GNind+GNpar+GNini)/double(GN));
if GN*count < (GNind+GNpar+GNini)
    count = count+1;
elseif GN*count > (GNind+GNpar+GNini)
    count = 1;
end
GN2 = GN;
SBstates = SBstructure.states(Ndep+1:end);

countGN1=0; countGN3=0;
for n=1:Npar
    if param.index(n)==1
        countGN1 = countGN1+1;
        GNcount1(countGN1,1)=n;
    end
end
for l=1:Nini
    if ic.index(l)==1
        countGN3 = countGN3+1;
        GNcount3(countGN3,1)=l;
    end
end
if countGN1~=0
countGN2=0;
for m=1:countGN1
    for n=1:Nini
        countGN2 = countGN2+1;
        dfdz1(1,countGN2)=dfdz(Nini*(GNcount1(m,1)-1)+n);
    end
end
end
if countGN3~=0
countGN4=0;
for m=1:countGN3
    for n=1:Nini
        countGN4 = countGN4+1;
        dfdz2(1,countGN4)=dfdz(Nini*Npar+Nini*(GNcount3(m,1)-1)+n);
    end
end
end

if countGN1~=0 && countGN3~=0
    dfdz=[dfdz1 dfdz2];
elseif countGN1==0 && countGN3~=0
    dfdz=dfdz2;
elseif countGN1~=0 && countGN3==0
    dfdz=dfdz1;
elseif countGN1==0 && countGN3==0

end

flag = 0;
if isempty(initial)~=1
    Fnames = fieldnames(initial);
    iccount = length(Fnames)-1;
    newdfdz = dfdz;
else
    iccount = 1; flag = 1;
end

%simulation
c = 1;
DC = 1;
DC1 = 1;
Stateflag=0;
Nalg = length(algebraic.name);
DSdata = cell(1,(GNind+GNpar+GNini));
fprintf('simulation starting time %fsec\n', toc);

for ICc=1:iccount
    if iccount >= 1 && flag == 0
        dfdz = newdfdz;
        GN = GN2;
    end
    
    for DSi=1:count
        if DSi == count
            GN2 = GN;
            GN = (GNind+GNpar+GNini) - GN*(DSi-1);
        end
        SBstructure.states(Ndep+1:end) = [];
        SBstructure.states(Ndep+1:(GN+1)*Ndep) = SBstates((DSi-1)*GN*Ndep+1:DSi*GN*Ndep);

        if ICc >=1 && flag == 0
            for i=1:length(initial)
                icname = sprintf('initial(i).%s;', Fnames{1});
                newicname = eval(icname);
                for j=1:Ndep
                    if strcmp(ic.symbol(j), newicname) == 1
                        icvalue = sprintf('initial(i).%s;', Fnames{ICc+1});
                        newicvalue = eval(icvalue);
                        SBstructure.states(i).initialCondition = newicvalue;
                        break;
                    end
                end
            end
        end
        

        [model] = DSmodel(SBstructure);
        [Fname1,Fname2]=DSmakeODEfile(model, Ndep, Nind, Nalg, GN);
        

        Si = zeros(1,length(model.states));
        for i=1:length(model.states)
            Si(1,i)=model.states(i).initialCondition;
        end
        
        dfdzFlag = 0;
        for i=1:length(model.variables)
            if ~strfind(model.variables(i).name,'dfdz_W')
                dfdzFlag = 1;
                break;
            end
        end
        clear model;
                
        T = zeros(length(timevector),1);
        data_int2 = zeros(length(timevector),Ndep*(GN+1));
        if dfdzFlag==0
            eval(sprintf('[T,data_int2] = %s(@%s,timevector,Si,options);',ode,Fname1));
        end
        
        if DSi==1 || Stateflag==0
            data.StateValue = data_int2(:,1:Ndep);
            data.time = T;
            Stateflag=1;
        end

        data_int2(:,1:Ndep) = [];
        for i=1:GN
            DSdata{DC} = data_int2(:,(i-1)*Ndep+1:i*Ndep);
            DC = DC+1;
        end
        
        clear data_int2;
        clear T;
        dfdz(1:GN*Ndep) = [];
        
        if rem(DSi,10) == 0 || DSi==count
            fprintf('simulation elapsed time %fsec %f%%\n',toc,(double(DSi)/double(count))*100);
        end
        
        delete(sprintf('%s/%s.m',pwd,Fname2));
        delete(sprintf('%s/%s.m',pwd,Fname1));
        clear Fname1 Fname2;
    end

    for i=1:GNind
        for j=1:Nind
            if independent.index(j) == 1
                data.DynamicSensitivity.independent.name{c} = independent.symbol(j);
                c=c+1;
            end
        end
        data.DynamicSensitivity.independent.value{i} = DSdata{DC1}(:,1:Ndep);
        DC1=DC1+1;
        c=1;
    end
    
    if length(DSdata) < (GNind+GNpar)
        error('Calculate Error');
    end
    
    for i=1:GNpar
        for j=1:Npar
            if param.index(j) == 1
                data.DynamicSensitivity.KineticParameter.name{c} = param.symbol(j);
                c=c+1;
            end
        end
        data.DynamicSensitivity.KineticParameter.value{i} = DSdata{DC1}(:,1:Ndep);
        DC1=DC1+1;
        c=1;
        
    end
    if length(DSdata) < (GNind+GNpar+GNini)
        error('Calculate Error');
    end
    for i=1:GNini
        for j=1:Nini
            if ic.index(j) == 1
                data.DynamicSensitivity.States.name{c} = ic.symbol(j);
                c=c+1;
            end
        end
        data.DynamicSensitivity.States.value{i} = DSdata{DC1}(:,1:Ndep);
        DC1=DC1+1;
        c=1;
    end
    data.StateName = ic.symbol;
    data.ParameterName = param.symbol;
    data.ParameterValue = param.value;
    
    if iccount >= 1 && flag == 0
        data.InitialCondition(1,:) = ic.symbol;
        for i=1:length(ic.symbol)
            data.InitialCondition{2,i} = SBstructure.states(i).initialCondition;
        end
        Initial.InitialCondition{ICc} = data;

        if ICc==iccount
            clear data;
            data = Initial;
        end
    end
end


fprintf('simulation endtime %fsec\n', toc);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                DSmodel                                  % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function [SBstructure] = DSmodel(SBstructure)        
        Vlength = length(SBstructure.variables);
        for Wi=1:Ndep*GN
            SBstructure.variables(Vlength+Wi).name = sprintf('dfdz_W%d', Wi);
            SBstructure.variables(Vlength+Wi).formula = char(dfdz(Wi));
        end
    end
end

